#!/bin/sh

declare -A msg

escReset="\033[2J\033[H\033[0m\e[?25h"          # Clear Home - Reset - Cursor "On"
escLf="\033[1B\033[80D" # Emulate CrLf
escClr="\033[2J\033[H"  # Clear Home
escInv="\033[7m"        # Inverse
escRst="\033[0m"        # Reset
escOff="\e[?25l"        # Cursor off
escOn="\e[?25h"         # Cursor on

HomeCurX=1
HomeCurY=22

msgCount=0
msgMax=10
msgX=3
msgY=11

lenMsg=64
lenLog=68

# ----------------------------------------------------------
# --- Print Message to Terminal "below" Mask and increment Cursor Y-Pos
# --- $1 = Message
escDebug()
{
    if [ $avrStatus -eq 1 ]; then return; fi
    # --- Move Cursor to Y:X and echo Message
    echo -n -e "\033[$HomeCurY"";""$HomeCurX""H$1"
    HomeCurY=$[HomeCurY+1]
}

# ----------------------------------------------------------
# --- echo Message(s) to Terminal "and" $LOGFILE
# --- $1 = Message / Outputstring
# --- $2 = Second Message
escEcho()
{
    local Arg

    if [ "$#" -gt 1 ]
    then
        escLog "| $1" "$2"
        escMsg "$1" "$2"
    else
        escLog "| $1"
        escMsg "$1"
    fi
}

# ----------------------------------------------------------
# --- echo Messages to Terminal or LOGFILE
# --- $1 = Message / Outputstring
# --- $2 = Second Message
escLog()
{
    local tLen
    local tLog

    if [ "$#" -gt 1 ]
    then
        # Bring string to length
        tLen=$[lenLog-${#2}-2]
        tLog=${1:0:${tLen}}
        tLen=$[lenLog-${#tLog}-${#2}-2]
        tLog="$(printf "%s%${tLen}s%s" "$tLog" " " "$2")"
    else
        # Bring string to length
        tLog=${1:0:${lenLog}}
        tLen=$[lenLog-${#tLog}]
        tLog="$(printf "%s%${tLen}s" "$tLog" " ")"
    fi
    
    if [ $avrInfo -eq 0 ];
        # --- In INFO-Mode "keine" Einträge in $LOGFILE
        then echo "$(date '+%Y.%m.%d %H:%M:%S') $tLog" >> "$LOGFILE"
        # --- Ausgabe im Terminal "nur" wenn AvrLogger im Hintergrund läuft
        else if [ $avrStatus -eq 0 ]; then echo "$tLog"; fi
    fi
}

# ----------------------------------------------------------
# --- Prints (echo) MSG to Maskbox
# --- "Niemals" direkt aufrufen!
# --- $1 = MSG
escMsg()
{
    local tLen
    local tMsg
    local i

    if [ "$#" -gt 1 ]
    then
        # Bring string to length
        tLen=$[lenMsg-${#2}]
        tMsg=${1:0:${tLen}}
        tLen=$[lenMsg-${#tMsg}-${#2}]
        tMsg="$(printf "%s%${tLen}s%s" "$tMsg" " " "$2")"
    else
        # Bring string to length
        tMsg=${1:0:${lenMsg}}
        tLen=$[lenMsg-${#tMsg}]
        tMsg="$(printf "%s%${tLen}s" "$tMsg" " ")"
    fi

    if [ $msgCount -lt $msgMax ]
    then
        msg[$msgCount]="$tMsg"
        msgCount=$[msgCount+1]
    else
        for (( i=0; i<msgMax-1; i++ )) do
            msg[$i]="${msg[$[i+1]]}"
        done
        msg[$[msgCount-1]]="$tMsg"
    fi

    for (( i=0; i<msgCount; i++ )) do
        escPrint $msgX $[msgY+i] "${msg[$i]}"
    done

    if [[ $avrInfo -eq 0 && $avrStatus = 0 ]]; then
        echo -n -e "\033[$HomeCurY"";""$HomeCurX""H" # Move Cursor to Y:X
    fi
}

# ----------------------------------------------------------
# --- Log Daten Formatieren und in der MaskBox ausgben.
# --- $1 = Log-Daten
escData()
{
    if [[ $avrStatus -eq 1 || $avrInfo -eq 1 ]]; then return 0; fi

    local dat="$(date '+%H:%M:%S')$1"

    dat=($dat)
    dat[0]=${dat[0]}                            # Time String
    dat[1]=$((${dat[1]}/10))                    # Load Int /10
    dat[2]=$(awk '{print $0/10}' <<< ${dat[2]})  # CPU  Float / 10
    dat[3]=$(awk '{print $0/10}' <<< ${dat[3]})  # Stat Float / 10
    dat[4]=$(awk '{print $0/10}' <<< ${dat[4]})  # Room Float / 10
    dat[5]=$(awk '{print $0/10}' <<< ${dat[5]})  # HDD  Float / 10
    dat[6]=$(awk '{print $0/10}' <<< ${dat[6]})  # PWM  Float / 10
    dat[7]=$(awk '{print $0/10}' <<< ${dat[7]})  # i2c  Float / 10
    dat[8]="$timerLap"

    escMsg "$(printf "%-8s  %3d  %5.1f  %5.1f  %5.1f  %5.1f  %5.1f  %5.1f   %6s" \
    ${dat[0]} ${dat[1]} ${dat[2]} ${dat[3]} ${dat[5]} ${dat[4]} ${dat[6]} ${dat[7]} ${dat[8]})"
}

# ----------------------------------------------------------
# --- Prints (echo) MSG at X-Y
# --- $1 = X-Pos
# --- $2 = Y-Pos
# --- $3 = MSG
escPrint()
{
    if [[ $avrStatus -eq 1 || $avrInfo -eq 1 ]]; then return 0; fi
    echo -n -e "\033[""$2"";""$1""H""$3"
    echo -n -e "\033[$HomeCurY"";""$HomeCurX""H" # Move Cursor to Y:X
}

# ----------------------------------------------------------
# --- Echo Terminal-Mask
escMask()
{
    if [[ $avrStatus -eq 1 || $avrInfo -eq 1 ]]; then return 0; fi

    local msgClear="$(printf "%${lenMsg}s" " ")"
    local temp

    echo -n -e $escClr
    echo -n -e "+------------------------------------------------------------------+"$escLf
    echo -n -e "| AvrVer. = ##.##.##  SvgVer. = ##.##.##  UpTime =   ####:##:##:## |"$escLf
    echo -n -e "| AvrTick =     ###s  SvgTick =     ###s  CPU    =   ############# |"$escLf
    echo -n -e "| AvrLog  =      ###  SvgLog  =      ###  Clock  = #(##) x #.##Ghz |"$escLf
    echo -n -e "+------------------------------------------------------------------+"$escLf
    echo -n -e "| Disk's   = ##  FAN Sens. = ##  ENV  = OFF   IP = ###.###.###.### |"$escLf
    echo -n -e "| CPU Sen. = ##  i2c Sens. = ##  Name = ########################## |"$escLf
    echo -n -e "| ---------------------------------------------------------------- |"$escLf
    echo -n -e "| Time     Load   CPU    Stat   HDD    Room   PWM    i2c     Loop  |"$escLf
    echo -n -e "| ---------------------------------------------------------------- |"$escLf
    echo -n -e "| 00:00:00    1   44.8   41.0   41.0   25.7   84.0   48.8   0.632s |"$escLf
    echo -n -e "| ##:##:##  ###  ###.#  ###.#  ###.#  ###.#  ###.#  ###.#  ##.#### |"$escLf
    echo -n -e "| ##:##:##  ###  ###.#  ###.#  ###.#  ###.#  ###.#  ###.#  ##.#### |"$escLf
    echo -n -e "|                                                                  |"$escLf
    echo -n -e "|                                                                  |"$escLf
    echo -n -e "|                                                                  |"$escLf
    echo -n -e "|                                                                  |"$escLf
    echo -n -e "|                                                                  |"$escLf
    echo -n -e "| ##:##:##  ###  ###.#  ###.#  ###.#  ###.#  ###.#  ###.#  ##.#### |"$escLf
    echo -n -e "| ##:##:##  ###  ###.#  ###.#  ###.#  ###.#  ###.#  ###.#  ##.#### |"$escLf
    echo -n -e "+------------------------------------------------------------------+"$escLf
#   echo -n -e "----->  [ ESC ]  -  [ Log ]  -  [ + ]  -  [ - ]  -  [ Help ]  <-----"$escLf

    for (( i=0; i<msgMax; i++ )) do
        msg[$i]="$msgClear"
        escPrint $msgX $[msgY+i] "${msg[$i]}"
    done
    # --- Print some Labels
    escPrint  3 1 $escInv" AvrLogger "$escRst
    escPrint 43 1 $escInv" "${SynoModel[1]}" "$escRst
    escPrint  3 5 $escInv" SENSORS "$escRst
    escPrint 34 5 $escInv" Environment "$escRst
    # --- Print some Values
    escPrint 13 2 "$AvrVersion"
    escPrint 33 2 "$SvgVersion"
    escPrint 54 2 "$(printf "%13.13s" ${SynoModel[4]})"
    escPrint 17 3 "$(printf "%3d" $AvrTick)"
    escPrint 37 3 "$(printf "%3d" $SvgTick)"
    escPrint 54 3 "$(printf "%13.13s" ${SynoCPU[2]})"
    escPrint 18 4 "$(printf "%3.3s" "$AvrLogStatus")"
    escPrint 38 4 "$(printf "%3.3s" "$SvgStatus")"
    temp=${SynoCPU[3]}"("${SynoCPU[4]}") x "$(printf "%1.2fGHz" ${SynoCPU[5]})    
    escPrint 52 4 "$(printf "%15.15s" "$temp")"
#   escPrint 14 6 "`printf "%2.2s" $SynoDisks`"
    escPrint 30 6 "$(printf "%2.2s" $fan_count)"
    escPrint 41 6 "$(printf "%3.3s" "$AvrEnvStatus")"
    escPrint 52 6 "$(printf "%15.15s" "$AvrEnvIp")"
    escPrint 14 7 "$(printf "%2.2s" $cpu_count)"
    escPrint 30 7 "$(printf "%2.2s" $i2c_count)"
    escPrint 41 7 "$(printf "%-27.27s" "$AvrEnvName")"

    echo -n -e "\033[$HomeCurY"";""$HomeCurX""H" # Move Cursor to Y:X
}

# ----------------------------------------------------------
# --- Only some examples for Copy & Paste
escTest()
{
    echo -n -e "\033[1B\033[80D"        # Emulate CrLf
    escLf="\033[1B\033[80D"             # Emulate CrLf

    echo -n -e "\033[s"                 # Save Cursor Position
    echo -n -e "\033[u"                 # Restore Cursor Position

    echo -n -e "\033[2J\033[H"          # Clear Home
    escClr="\033[2J\033[H"              # Clear Home

    echo -n -e "\033[7m"                # Inverse
    escInv="\033[7m"                    # Inverse

    echo -n -e "\033[0m"                # Reset
    escRst="\033[0m"                    # Reset

    echo -n -e "\033[2;5H"              # Move Cursor to Y ; X

    echo -n -e "\e[?25l"                # Cursor off
    escOff="\e[?25l"                    # Cursor off

    echo -n -e "\e[?25h"                # Cursor on
    escOn="\e[?25h"                     # Cursor on
}

#   l=${#1}
#   printf "Zahl 67: |%-5i|\n" 67 #linksbuendig
#   temp=`printf "%-${msgLen}s" "$1"`
#   msg[$msgCount]="$1"
#   printf "%sDisk%3d | %-5s | %-22s | %2d°C\n" "$stamp" $id "$ty" "$mo" $te
#   echo $(xxd -pu <<< "$IFS")
