# ----------------------------------------------------------
# --- Trap STRG-C Handle
# ----------------------------------------------------------
STRG_C()
{
    if [ $avrTest -eq 1 ]; then
        TimerStart
        Svg $addNewLines
        TimerStop
        escEcho "AVR: TEST-Job completed" "$timerLap"
    fi

    escLog "+------------------------------------------------------------------+"
    if [ $avrStatus -eq 0 ]
        then escLog "| AVR: AvrLogger was terminated by the User                        |"
        else escLog "| AVR: AvrLogger was stopped by System: ShutDown                   |"
    fi
    escLog "+--------------------------= TERMINATED =--------------------------+"

    if [[ $avrStatus -eq 0 && $avrInfo -eq 0 ]]; then
        echo -n -e "\033[$HomeCurY"";""$HomeCurX""H" # Move Cursor to Y:X
    fi

    if [ $avrStatus -eq 0 ]; then
        stty sane
        echo -e "\e[?25hAvrLogger aborted by User"
    fi

    if [[ -e "$workingDir""avr.pid" && $avrExecution -eq 0 ]]; then rm "$workingDir""avr.pid"; fi
    exit 1
}

# ----------------------------------------------------------
# Cut a file from the end to numbered lines
# ----------------------------------------------------------
# --- $1 = File
# --- $2 = Number of Lines
trimFile()
{
    local n

	if [ ! -e "$1" ]; then return; fi

    # ----------------------------------------------------------
    # --- Get number of lines; if $n <= Numer of Lines; Return
    n=$(wc -l "$1" | cut -d" " -f1)
    if [ $n -le $2 ]; then return; fi

    n=$[n-$2]
    sed -i "1,${n}d" "$1"
}

# ----------------------------------------------------------
# Divide given avr.dat back to Array
# ----------------------------------------------------------
declare -A pStat

initPlotData()
{
    local Range

    TimerStart

    Range=$[$(GetValue "$SvgConf" "VerMax")-$(GetValue "$SvgConf" "VerMin")]
    VerScaleX=$[(pw*10)/Range]

    # --- Plot Parameters ---
    y_Min[2]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"     'PlotLoadAxis')'Min'); y_Max[2]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"     'PlotLoadAxis')'Max')
    y_Min[3]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"      'PlotCPUAxis')'Min'); y_Max[3]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"      'PlotCPUAxis')'Max')
    y_Min[4]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"   'PlotStatusAxis')'Min'); y_Max[4]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"   'PlotStatusAxis')'Max')
    y_Min[5]=$(GetValue "$SvgConf" $(GetValue "$SvgConf" 'PlotAmbienteAxis')'Min'); y_Max[5]=$(GetValue "$SvgConf" $(GetValue "$SvgConf" 'PlotAmbienteAxis')'Max')
    y_Min[6]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"      'PlotHddAxis')'Min'); y_Max[6]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"      'PlotHddAxis')'Max')
    y_Min[7]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"      'PlotPwmAxis')'Min'); y_Max[7]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"      'PlotPwmAxis')'Max')
    y_Min[8]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"      'Ploti2cAxis')'Min'); y_Max[8]=$(GetValue "$SvgConf" $(GetValue "$SvgConf"      'Ploti2cAxis')'Max')

    for ((i=2;i<=8;i++)) do
        y_Range[$i]=$[y_Max[$i]-y_Min[$i]]
        y_Min[$i]=$[y_Min[$i]*10]
        y_Scale[$i]=$[(ph*10)/y_Range[$i]]
    done

    trimFile "$AvrDat" 1440

    csvRow=0; csvCol=0

	if [ ! -e "$AvrDat" ]
	then
	    svgTimeline=0
	else
        minRow=1
    	for temp in `cat "$AvrDat"`;do
        	plotData[$csvRow,$csvCol]=$temp
        	if [ $csvCol -lt $TabLast ]
        	then
        	    csvCol=$[csvCol+1]
        	else
            	plotData[$csvRow,0]=$[(VerScaleX*csvRow)/10]
                for ((i=2;i<=8;i++)) do
                    plotData[$csvRow,$i]=$[((${plotData[$csvRow,$i]}-${y_Min[$i]})*${y_Scale[$i]})/100]
                done
        	    csvRow=$[csvRow+1]
        	    csvCol=0
        	fi
    	done
	    csvRow=$[csvRow-1]
	fi
    csvCol=$TabLast

    if [ $csvRow -eq 1439 ]; then maxRow=1; fi

    svgTimeline=$[$(date +%s)-((csvRow+1)*AvrTick)]
    svgTimeline="$(date -d @"$svgTimeline" '+%H:%M')"
    svgTimeline=$(TimeToSec "$svgTimeline")

    TimerStop
    if [ $csvRow -eq 0 ]
        then escEcho "AVR: No Plot-Data imported"
        else escEcho "AVR: Plot-Data imported : $[csvRow+1]" "$timerLap"
    fi
}

# ----------------------------------------------------------
# If necessary, shift Array content one position down;
# Add a new Row to plotData-Array.
# --- $1 = New Content
# ----------------------------------------------------------
minRow=0
maxRow=0
addPlotData()
{
    local dat=($1)
    local i

    if [ $maxRow -eq 1 ]; then
        # ----------------------------------------------------------
        # --- Shift Array if necessary
        for ((i=0;i<=csvRow-1;i++)) do
            for ((j=1;j<=csvCol;j++)) do
    	        plotData[$i,$j]=${plotData[$[i+1],$j]}
            done
        done
    fi
    # ----------------------------------------------------------
    if [ $minRow -eq 0 ]
    then
        minRow=1
    else
        if [[ $maxRow -eq 0 && $csvRow -lt 1439 ]]
            then csvRow=$[csvRow+1]
            else maxRow=1
        fi
    fi
    # ----------------------------------------------------------
    plotData[$csvRow,0]=$[(VerScaleX*csvRow)/10]
    plotData[$csvRow,1]=${dat[1]}
    for ((i=2;i<=8;i++)) do
        plotData[$csvRow,$i]=$[((${dat[$i]}-${y_Min[$i]})*${y_Scale[$i]})/100]
    done
    # ----------------------------------------------------------

    svgTimeline=$[$(date +%s)-((csvRow+1)*AvrTick)]
    svgTimeline="$(date -d @"$svgTimeline" '+%H:%M')"
    svgTimeline=$(TimeToSec "$svgTimeline")

    addNewLines=$[addNewLines+1]
}

# ----------------------------------------------------------
# --- Add a New Line to File. Limit number of lines.
# --- $1 = New Line MSG
# --- $2 = Destination-File
# --- $3 = Number of the line to be deleted
# --- $4 = Max Linenumbers in Destination-File
addLine()
{
    local n

    if [ ! -e "$2" ]
    then
        echo "$1" > "$2"
    	chmod 666 "$2"
    	return
    else
        echo "$1" >> "$2"
    fi

    if [ $3 -gt 0 ]; then
        n=$(wc -l "$2" | cut -d" " -f1)

        if [ $n -le $((${4}+${3}-1)) ]; then return; fi

        n=$[(n-(${4}+${3}-1))+${3}-1]
        sed -i "${3},${n}d" "$2"
    fi
}
# ----------------------------------------------------------
# --- Copy "single" quoted Key to "all" nonquoted Key's
# --- $1 = Source-File
# --- $2 = Destination-File
# --- $3 = Key
# ----------------------------------------------------------
CpySvg()
{
    sed -i 's|'"$3"'|'"$(GetValue "$1" "$3")"'|g' "$2"
}

# ----------------------------------------------------------
# --- Change "any" Key's (without quoting)
# --- $1 = File
# --- $2 = Key
# --- $3 = Value
# ----------------------------------------------------------
PutSvg()
{
    sed -i 's|'"$2"'|'"$3"'|g' "$1"
}

# ----------------------------------------------------------
# --- Change a "single" Key quoted between "
# --- $1 = File
# --- $2 = Key
# --- $3 = Value
# ----------------------------------------------------------
PutValue()
{
    sed -i '/^'"$2"'.*$/ s|\(\"\).*\(\"\)|\1'"$3"'\2|' "$1"
}

# ----------------------------------------------------------
# --- Get SMART-Value(s) fron all HDD's
# --- $1 = SMART-Key
# ----------------------------------------------------------
getSMART()
{
    local mx
    local bo
    local smart
	mx=""
	for bo in `/bin/ls /dev/sd?`
	do
		smart=`/usr/bin/smartctl -d ata -A $bo`
		mx+=`echo "$smart" | grep "$1" | awk '{print $10" "}'`
	done
	echo $mx
}

# ----------------------------------------------------------
# --- Get Time in nanoseconds
TimerStart()
{
    timerStart[$timerCount]=$(date +%s%N)
    timerCount=$[timerCount+1]
}

# ----------------------------------------------------------
# --- Echo passed time in milliseconds
TimerStop()
{
    timerCount=$[timerCount-1]

    timerLap=$(($(date +%s%N)-${timerStart[$timerCount]}))
    timerLap=$(awk '{print $0/1000000000}' <<< $timerLap)
    timerLap="$(printf "%6.3f" $timerLap)"
}

# ----------------------------------------------------------
# Calculate the seconds from given String "hh:mm"
# --- $1 = Time formatet String
# ----------------------------------------------------------
TimeToSec()
{
    echo $[($(echo ${1:0:2} | sed 's/^0//')*3600) + ($(echo ${1:3:2} | sed 's/^0//')*60)]
}

# ----------------------------------------------------------
# --- 3-Byte Integer als Festkomawert ausgeben
# ----------------------------------------------------------
prt_2d1()
{
    local t
    t=$1
	echo $ColSign${t:0:2}$DecSign${t:2:1}
}

# ----------------------------------------------------------
# --- Schlüssel aus Config-File lesen
# --- $1 = Config-File
# --- $2 = Key
# ----------------------------------------------------------
GetValue()
{
    local line
    if [ -e "$1" ]
    then
        line="$(grep -m 1 "^$2" "$1")"
        if [ ${#line} -gt 0 ]
        then
            echo "$(sed 's/^[^"]*"\([^"]*\)".*/\1/g' <<< "$line")"
        else
            escEcho  "ERROR: '$2' in '$1' not fount"
            escEcho  "----------------------------------------------------------------"
            exit
        fi
    else
        escEcho  "ERROR: '$1' not fount"
        escEcho  "----------------------------------------------------------------"
        exit
    fi
}

# ----------------------------------------------------------
# --- Moving Avarage Array initialisieren
InitArray()
{
	arr_row=$1	# 10
	arr_col=$2	# 10
	arr_cnt=0

	for ((i=0;i<=arr_row;i++)) do
		for ((j=0;j<=arr_col;j++)) do
	    	arr[$i,$j]=0
		done
	done
}

